@@ -1,6 +1,7 @@
 #! /usr/bin/python3.9
 "Python script containing functions and classes for general cryptographic use."
 
+# importing libraries
 import os
 import rsa
 import string
@@ -47,7 +48,14 @@ class InvalidKeyArgument(Exception):
     pass
 
 
-def hash_(StringToHash:str, hash_code:str, return_type="hex") -> str: # tested
+class UnknownError(Exception):
+    """
+    Raised when a an unknown error occurrs during encryption/decryption.
+    """
+    pass
+
+
+def hash_(StringToHash:str, hash_code:str, return_type="hex"): # tested
     """
     Miscellenious function for implementing hash algorithms.
 
@@ -69,25 +77,25 @@ def hash_(StringToHash:str, hash_code:str, return_type="hex") -> str: # tested
     """
     if hash_code == "SHA224":
         hash_obj = hash.sha224(StringToHash.encode())
-        if return_type == "bytes":
+        if return_type.lower() == "bytes":
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
     if hash_code == "SHA256":
         hash_obj = hash.sha256(StringToHash.encode())
-        if return_type == "bytes":
+        if return_type.lower() == "bytes":
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
     if hash_code == "SHA512":
         hash_obj = hash.sha512(StringToHash.encode())
-        if return_type == "bytes":
+        if return_type.lower() == "bytes":
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
     if hash_code == "MD5":
         hash_obj = hash.md5(StringToHash.encode())
-        if return_type == "bytes":
+        if return_type.lower() == "bytes":
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
@@ -132,20 +140,20 @@ def token_generate(size:int, return_type="HEX"): # tested
 
     returns: a token of the specific type, or 1 to indicate that the return type was not valid.
     """
-    if return_type == "HEX":
+    if return_type.upper() == "HEX":
         token = secrets.token_hex(size)
         return token
-    if return_type == "BYTES":
+    if return_type.upper() == "BYTES":
         token = secrets.token_bytes(size)
         return token
-    if return_type == "URL":
+    if return_type.upper() == "URL":
         token = secrets.token_urlsafe(size)
         return token
     else:
         return 1
 
 
-def gen_random_password(length:int) -> str: # tested
+def generate_password(length:int) -> str: # tested
     """
     Generates and returns a random password of n `length`.
     """
@@ -156,6 +164,8 @@ def gen_random_password(length:int) -> str: # tested
         password.append(secrets.choice(characters))
     shuffle(password)
     final_password = "".join(password)
+    # deleting uneeded variables
+    del characters
     return final_password
 
 
@@ -169,10 +179,14 @@ def sec_delete(file_path:str, random_fill = True, null_fill = True, passes = 35)
     2: Attempt to pass a dangerous command to command line.
     0: File successfully deleted.
     """
-    if "/home/" not in file_path or "/." in file_path and get_platform=="Linux":
+    file_path = os.path.abspath(file_path)
+    if "/home/" not in file_path or "/." in file_path:
         return 1
     elif "sudo rm -rf /" in file_path:
         return 2
+    # testing if platform is Linux
+    if get_platform() != "Linux":
+        return 3
     else:
         with open (file_path, "rb") as file:
             data = file.read()
@@ -186,6 +200,8 @@ def sec_delete(file_path:str, random_fill = True, null_fill = True, passes = 35)
                 with open (file_path, "wb") as file:
                     file.write(b"\x00" * length)
         os.system("rm {}".format(file_path))
+        # deleting uneeded variables
+        del data, length, file_path, random_fill, null_fill, passes
         return 0
 
 
@@ -198,13 +214,17 @@ def delete(path:str) -> int: # tested
 
     Returns: An integer value indicating if the function successfully executed.
     """
+    path = os.path.abspath(path)
     if '/home/' not in path or "/." in path:
         return 1
     elif "sudo rm -rf /" in path:
         return 2
-    else:
+    # checking if the platform is Linux
+    elif get_platform() == "Linux":
         os.system("rm {}".format(path))
         return 0
+    else:
+        return 3
 
 
 def XOR(bytes1:bytes, bytes2:bytes, write_to=None):
@@ -213,10 +233,61 @@ def XOR(bytes1:bytes, bytes2:bytes, write_to=None):
     Returns: None if `write_to` is `None`, otherwise returns the XOR'ed string.
     """
     bytes_string = strxor(bytes1, bytes2, output = write_to)
-    if bytes_string is not None:
-        return bytes_string
-    else:
-        return None
+    return bytes_string
+
+
+def one_time_pad_encrypt(message:str, key:str):
+    """
+    A function for performing one time pad encryption. using a key generated from a keyphrase."""
+    # generating the key
+    key_bytes = hash_(key, "SHA256", return_type="bytes")
+    # getting the length of the key
+    key_length = len(key_bytes)
+    # getting the length of the message bytes
+    message_length = len(message.encode())
+    # coverting the message to bytes
+    print (key_length, message_length)
+    message_bytes = message.encode()
+    # ensuring message length is a multiple of key length
+    if message_length % key_length != 0:
+        message_bytes = message_bytes + b"\x00" * (key_length - (message_length % key_length))
+    #breaking the message into chunks
+    message_chunks = [message_bytes[i:i+key_length] for i in range(0, len(message_bytes), key_length)]
+    # XORing the chunks with the key
+    chunks = []
+    for i in range(len(message_chunks)):
+        XORed_chunks = XOR(message_chunks[i], key_bytes)
+        chunks.append(XORed_chunks)
+    assert len(chunks) == len(message_chunks)
+    encrypted_message = b"".join(chunks)
+    # deleting all variables
+    del key_bytes, key_length, message_length, message_chunks, XORed_chunks
+    # returning the XORed string
+    return encrypted_message
+
+
+def one_time_pad_decrypt(message:bytes, key:str):
+    """
+    A function for performing one time pad decryption. using a key generated from a keyphrase."""
+    # generating the key
+    key_bytes = hash_(key, "SHA256", return_type="bytes")
+    # getting the length of the key
+    key_length = len(key_bytes)
+    # breaking the message bytes into chunks of the key length
+    message_chunks = [message[i:i+key_length] for i in range(0, len(message), key_length)]
+    # XORing the chunks with the key
+    chunks = []
+    for i in range(len(message_chunks)):
+        XORed_chunks = XOR(message_chunks[i], key_bytes)
+        chunks.append(XORed_chunks)
+    # joining the chunks
+    message_bytes = b"".join(chunks)
+    # removing the null bytes
+    message_bytes = message_bytes.rstrip(b"\x00")
+    # deleting all variables
+    del key_bytes, key_length, message_chunks, XORed_chunks
+    # returning the XORed string
+    return message_bytes
 
 
 def is_prime_number(number:int) -> bool:
@@ -256,15 +327,11 @@ def englishToKey(words:str) -> bytes:
     if amount_of_words % 6 != 0:
         raise InvalidKeyArgument("Given amount of words are not a multiple of 6.")
     key = english_to_key(s=words)
+    # deleting uneeded variables
+    del amount_of_words, split_words
     return key
 
 
-def generate_key(password:str, salt:bytes, dkLen:int) -> bytes:
-    if len(password) == 0:
-        return 1
-    pass
-
-
 def keyToEnglish(key:bytes):
     """
     A function for  converting a bytestring to a string of english words.
@@ -278,8 +345,6 @@ def keyToEnglish(key:bytes):
     words = key_to_english(key=key)
     return words
 
-    pass
-
 
 class Ciphers: # tested
     """
@@ -300,7 +365,7 @@ class Ciphers: # tested
     # misc methods
 
 
-    def generate_symmetric_key(self, salt=None) -> 'tuple[bytes, bytes]': # tested
+    def generate_symmetric_key(self, salt=None or bytes) -> 'tuple[bytes, bytes]': # tested
         "Method for generating keys. Will return tuple (key, salt) if salt is not provided."
         if salt is None:
             salt_ = os.urandom(32)
@@ -325,53 +390,33 @@ class Ciphers: # tested
 
         Return codes:
             0: File path is suitable.
-            1: File path is None.
-            2: File path is invalid.
-            3: App does not have permission to access file.
-            4: File is hidden.
-            5: File is part of linux root filesystem.
-            6: File encryption marker detected during encryption.
-            7: File encryption marker not detected during decryption.
-            8: File was encrypted with a different algorithm.
+            1: File path is invalid.
         """
-        if path is None:
-            return 1
-        # checking if file is hidden
-        if "/." in path:
-            return 4
-        # checking if file is part of the Linux root filesystem
-        if get_platform() == 'Linux':
-            if "/home/" not in path:
-                return 5
-        # checking if decrypt is cancelled
         try:
-            if os.path.isfile(path) is True:
+            assert path is not None
+            assert "/." not in path
+            assert get_platform() == "linux" and "/home/" in path
+            assert os.path.isfile(path) is True or os.path.isdir(path) is True
+            try:
                 with open (path, "rb") as file:
-                    type_of_file = file.read(3)
-                    file_marker = file.read(14)
-            elif os.path.isdir(path) is True:
+                    file_type = file.read(3)
+                    file_marker = file.read(len(self.marker))
+            except PermissionError:
+                return 1
+            except FileNotFoundError:
+                return 1
+            except IsADirectoryError:
                 return 0
-        # checking if file exists
-        except FileNotFoundError:
-            return 2
-        # checking if application has permission
-        except PermissionError:
-            return 3
-        # checking if there is a file encryption marker during encryption
-        if file_marker == self.marker and decrypting is False:
-            return 6
-        # checking if there is not a file encryption marker during decryption
-        elif file_marker != self.marker and decrypting is True:
-            return 7
-        if decrypting is True:
-            if type_of_file.decode() != type_:
-                return 8
+            assert file_type == type_
+            if decrypting:
+                assert file_marker == self.marker
             else:
-                return 0
-        else:
-            return 0
-
+                assert file_marker != self.marker
+        except AssertionError:
+            return 1
+        return 0
     
+
     def change_encrypting_password(self, old_password:str, new_password:str):  # tested
         """
         A method for changing the classes' `password` attribute.
@@ -412,7 +457,7 @@ class Ciphers: # tested
         return decrypted_string
 
 
-        # AES methods
+    # AES methods
 
 
     def AES_encrypt_file(self, path:str) -> int: # tested
@@ -423,13 +468,8 @@ class Ciphers: # tested
             `path`: The path of the file/folder to be encrypted.
 
         Return codes:
-            0: File encrypt successful.
-            1: Path is None.
-            2: Path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of Linux root filesystem.
-            6: File encryption marker detected during encryption.
+            0: File encryption successful.
+            1: Path is invalid.
         """
         # checking if the file path is ok
         return_code = self.check_path_validity(path, decrypting=False, type_="AES")
@@ -437,8 +477,8 @@ class Ciphers: # tested
             return return_code
         try:
             AES_encrypt_file(self.password, path)
-        except Exception:
-            pass
+        except Exception as error:
+            raise UnknownError("Unknown error occurred: {}".format(error))
         return 0
 
 
@@ -450,14 +490,7 @@ class Ciphers: # tested
 
         return codes:
             0: File decrypt successful.
-            1: Path is None.
-            2: File path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of linux root filesystem.
-            6: File was encrypted with a different key.
-            7: File was not encrypted.
-            8: File was encrypted using a different alogorithm.
+            1: Path is invalid.
         """
         # checking if file is suitable
         return_code = self.check_path_validity(path, decrypting=True, type_="AES")
@@ -466,8 +499,8 @@ class Ciphers: # tested
         # passing path onto compiled Cython code
         try:
             AES_decrypt_file(self.password, path)
-        except Exception:
-            pass
+        except Exception as error:
+            raise UnknownError("Unknown error occurred: {}".format(error))
         return 0
 
 
@@ -478,15 +511,22 @@ class Ciphers: # tested
         Parameters:
             `string_to_encrypt`: the string being encrypted, in bytes
             `key`: the key to encrypt the string with. If not provided, will generate a new key using the `Ciphers` class `password` attribute.
-        returns a tuple of `(ciphered_string, iv, key)`
+        returns a tuple of `(ciphered_string, iv, key)` if key is not provided; otherwise returns a tuple of `(ciphered_string, iv)`.
         """
         if key is None:
             salt = os.urandom(32)
             key = self.generate_symmetric_key(salt)
+        try:
+            assert isinstance(string_to_encrypt, bytes)
+        except AssertionError:
+            string_to_encrypt = string_to_encrypt.encode()
         iv = os.urandom(16)
         cipher = AES.new(key, AES.MODE_CFB, iv=iv)
         ciphered_string = cipher.encrypt(string_to_encrypt)
-        return (ciphered_string, iv, key)
+        if key is None:
+            return (ciphered_string, iv, key)
+        else:
+            return (ciphered_string, iv)
 
 
     def AES_decrypt_string(self, encrypted_string:bytes, key:bytes, iv:bytes):  # tested
@@ -499,10 +539,10 @@ class Ciphers: # tested
             `iv`: The Intialization vector used for encryption.
         returns the decrypted string in bytes format.
         """
-        if key is None:
-            raise NoKeyError("No key was provided for the cipher.")
-        if iv is None:
-            raise InvalidCipherArgument("IV was not provided.")
+        try:
+            assert key is not None and iv is not None
+        except AssertionError:
+            raise InvalidCipherArgument("Key and IV must be provided.")
         cipher = AES.new(key, AES.MODE_CFB, iv=iv)
         decrypted_string = cipher.decrypt(encrypted_string)
         return (decrypted_string)
@@ -520,12 +560,7 @@ class Ciphers: # tested
 
         Return codes:
             0: File encrypt successful.
-            1: Path is None.
-            2: Path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of a Linux root filesystem.
-            6: File encryption marker detected during encryption.
+            1: Path is invalid.
         """
         # checking if file is suitable
         return_code = self.check_path_validity(path, decrypting=False, type_="BLO")
@@ -534,8 +569,8 @@ class Ciphers: # tested
         # passing onto cython code
         try:
             BLO_encrypt_file(self.password, path)
-        except Exception:
-            pass
+        except Exception as error:
+            raise UnknownError("Unknown error occurred: {}".format(error))
         return 0
 
 
@@ -548,21 +583,17 @@ class Ciphers: # tested
 
         return codes:
             0: File decrypt successful.
-            1: Path is None.
-            2: File path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of  a Linux root filesystem.
-            6: File was encrypted with a different key.
-            7: File was not encrypted.
-            8: File was encrypted using a different alogorithm.
+            1: Path is invalid
         """
         # checking if file is suitable
         return_code = self.check_path_validity(path, decrypting=True, type_="BLO")
         if return_code != 0:
             return return_code
-        # passsing onto cython code
-        BLO_decrypt_file(self.password, path)
+        # passing onto cython code
+        try:
+            BLO_decrypt_file(self.password, path)
+        except Exception as error:
+            raise UnknownError("Unknown error occurred: {}".format(error))
         return 0
 
 
@@ -578,10 +609,17 @@ class Ciphers: # tested
         if key is None:
             salt = os.urandom(32)
             key = self.generate_symmetric_key(salt)
+        try:
+            assert isinstance(string_to_encrypt, bytes)
+        except AssertionError:
+            string_to_encrypt = string_to_encrypt.encode()
         iv = os.urandom(8)
         cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
         ciphered_string = cipher.encrypt(string_to_encrypt)
-        return (ciphered_string, iv, key)
+        if key is None:
+            return (ciphered_string, iv, key)
+        else:
+            return (ciphered_string, iv)
 
 
     def BLO_decrypt_string(self, encrypted_string:bytes, key:bytes, iv:bytes):  # tested
@@ -594,10 +632,10 @@ class Ciphers: # tested
             `iv`: The Intialization vector used for encryption.
         returns the decrypted string in bytes format.
         """
-        if key is None:
-            raise NoKeyError("No key was provided for the cipher.")
-        if iv is None:
-            raise InvalidCipherArgument("No intialization vector was provided for the cipher.")
+        try:
+            assert key is not None and iv is not None
+        except AssertionError:
+            raise InvalidCipherArgument("Key and IV must be provided.")
         cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
         decrypted_string = cipher.decrypt(encrypted_string)
         return (decrypted_string)
@@ -618,16 +656,11 @@ class Ciphers: # tested
 
         returns:
             0: File encrypt successful.
-            1: Path is None.
-            2: Path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of Linux root filesystem.
-            6: File encryption marker detected during encryption.
+            1: Path is invalid.
         """
         # file encryption
         if os.path.isfile(path) is True:
-            #chyecking if file is suitable
+            #checking if file is suitable
             return_code = self.check_path_validity(path, decrypting=False, type_="FER")
             if return_code != 0:
                 return return_code
@@ -672,14 +705,7 @@ class Ciphers: # tested
 
         return codes:
             0: File decrypt successful.
-            1: Path is None.
-            2: File path is invalid.
-            3: App does not have permissions required to access file.
-            4: File is hidden.
-            5: File is part of root filesystem (linux Debian based distros only).
-            6: File was encrypted with a different key.
-            7: File was not encrypted.
-            8: File was encrypted using a different alogorithm.
+            1: Path is Invalid.
         """
         if key is None:
             raise NoKeyError("No key was provided for decryption")
@@ -724,7 +750,7 @@ class Ciphers: # tested
             return 0
     
 
-    def FerNet_encrypt_string(self, bstring=bytes, key=None):  # tested
+    def FerNet_encrypt_string(self, bstring:bytes, key=None):  # tested
         """
         Method for encrypting a string with the `cryptography` module's `Fernet` class.
 
@@ -735,8 +761,15 @@ class Ciphers: # tested
         if key is None:
             key = self.generate_FerNet_key()
         f = Fernet(key)
+        try:
+            assert isinstance(bstring, bytes)
+        except AssertionError:
+            bstring = bstring.encode()
         encrypted_text = f.encrypt(bstring)
-        return encrypted_text, key
+        if key is None:
+            return (encrypted_text, key)
+        else:
+            return (encrypted_text)
 
 
     def FerNet_decrypt_string(self, encrypted_string:bytes, key:bytes):  # tested
@@ -747,13 +780,25 @@ class Ciphers: # tested
 
         Will return the decrypted text as a `bytes` object.
         """
-        if key is None:
-            raise NoKeyError("No key was provided for the cipher.")
+        try:
+            assert key is not None
+        except AssertionError:
+            raise NoKeyError("No key was provided for decryption")
         f = Fernet(key)
         decrypted_text = f.encrypt(encrypted_string)
         return decrypted_text
 
 
+    def change_file_marker(self, new_file_marker:bytes):
+        """
+        Method for changing the file marker used to identify encrypted files.
+
+        ## THIS MAY PREVENT DECRYPTION OF FILES THAT WERE ENCRYPTED WITH ANOTHER FILE MARKER.
+        """
+        self.marker = new_file_marker
+        return 0
+
+
 class Cipher_Constructor:
     """
     A class for simplifying cipher object generation.
@@ -1268,4 +1313,180 @@ class Cipher_Constructor:
         if key is None:
             raise NoKeyError("Key was not provided.")
         fernet_cipher = Fernet(key=key)
-        return fernet_cipher
\ No newline at end of file
+        return fernet_cipher
+
+
+def init_diagonostic():
+    """A method for testing the various functions of this library."""
+    print("Testing functions in the `cipher` module.")
+    print ("testing hashes")
+    assert hash_("Hello World", "SHA256") == hash.sha256("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA512") == hash.sha512("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA224") == hash.sha224("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "MD5") == hash.md5("Hello World".encode("utf-8")).hexdigest()
+    print ("Hexdigests match")
+    print("testing byte hashing")
+    assert hash_("Hello World", "SHA256", return_type="bytes") == hash.sha256("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA512", return_type="bytes") == hash.sha512("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA224", return_type="bytes") == hash.sha224("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "MD5", return_type="bytes") == hash.md5("Hello World".encode("utf-8")).digest()
+    print ("Bytes match")
+    print ("testing key generation")
+    print ("generating length 32 password")
+    password = generate_password(32)
+    print ("password:", password)
+    # assert that length is 32
+    assert len(password) == 32
+    print ("generating length 64 password")
+    password = generate_password(64)
+    print ("password:", password)
+    # assert that length is 64
+    assert len(password) == 64
+    print ("generating length 128 password")
+    password = generate_password(128)
+    print ("password:", password)
+    # assert that length is 128
+    assert len(password) == 128
+    print ("test complete; key generation passed")
+    # testing hash comparision function
+    print ("testing hash comparision function")
+    assert compare_hashes(hash_("Hello World", "SHA256"), hash_("Hello World", "SHA256")) == True
+    assert compare_hashes(hash_("Hello World", "SHA256"), hash_("Not Hello World", "SHA512")) == False
+    print ("test complete; hash comparision function passed")
+    # testing random choice function
+    print ("testing random choice function")
+    assert random_choice(["Hello", "World", "!"]) == "Hello" or "World" or "!"
+    print ("test complete; random choice function passed")
+    # testing token generation function
+    print ("testing token generation function")
+    bytes_token = token_generate(32, return_type="BYTES")
+    print ("bytes token:", bytes_token)
+    assert len(bytes_token) == 32
+    hex_token = token_generate(32, return_type="HEX")
+    print ("hex token:", hex_token)
+    assert len(hex_token) == 64
+    url_safe_token = token_generate(32, return_type="URL")
+    print ("url safe token:", url_safe_token)
+    print (len(url_safe_token))
+    assert len(url_safe_token) == 43
+    print ("test complete; token generation function passed")
+    print ("testing secure file deletion function")
+    # testing secure file deletion function
+    #creating a test file
+    test_file = open("test_file.txt", "w")
+    test_file.write("Hello World")
+    test_file.close()
+    # deleting using secure file deletion function
+    exit_code = sec_delete("test_file.txt")
+    print(exit_code)
+    # checking if file is deleted
+    assert not os.path.exists("test_file.txt")
+    print ("test complete; secure file deletion function passed")
+    print ("testing file deletion function")
+    # testing file deletion function
+    #creating a test file
+    test_file = open("test_file.txt", "w")
+    test_file.write("Hello World")
+    test_file.close()
+    # deleting using file deletion function
+    delete("test_file.txt")
+    # checking if file is deleted
+    assert not os.path.exists("test_file.txt")
+    print ("test complete; file deletion function passed")
+    print ("testing XOR function")
+    # testing XOR function against pycryptodome's XOR function
+    assert XOR(b"Hello World", b"Hello World") == strxor("Hello World".encode("utf-8"), "Hello World".encode("utf-8"))
+    print ("test complete; XOR function passed")
+    # testing one time pad encrypt
+    print ("testing one time pad encrypt/decrypt")
+    encrypted_string = one_time_pad_encrypt("Hello World", "youwillneverguess")
+    print ("encrypted string:", encrypted_string)
+    decrypted_string = one_time_pad_decrypt(encrypted_string, "youwillneverguess")
+    print ("decrypted string:", decrypted_string)
+    assert decrypted_string == b"Hello World"
+    print ("test complete; one time pad encrypt/decrypt passed")
+    # testing the is_prime_number function
+    print ("testing is_prime_number function")
+    assert is_prime_number(2) == True
+    assert is_prime_number(3) == True
+    assert is_prime_number(4) == False
+    assert is_prime_number(5) == True
+    assert is_prime_number(6) == False
+    print ("test complete; is_prime_number function passed")
+    # testing the generate_prime_number function
+    print ("testing generate_prime_number function")
+    assert get_prime_number(2) == 2 or 3 or 5 or 7
+    print ("test complete; generate_prime_number function passed")
+    # testing the GCD function
+    print ("testing GCD function")
+    assert GCD(2, 4) == 2
+    assert GCD(4, 2) == 2
+    assert GCD(2, 6) == 2
+    assert GCD(6, 2) == 2
+    print ("test complete; GCD function passed")
+    print ("Functions test passed | Function tests passed")
+    print ("Testing Ciphers Class")
+    print ("testing RSA methods")
+    # instantiating the class
+    cipher = Ciphers("youwillneverguess")
+    # generating a key
+    public_key, private_key = cipher.generate_RSA_keys()
+    # encrypting a string
+    encrypted_string = cipher.RSA_encrypt_str(public_key, "Hello World")
+    # decrypting the string
+    decrypted_string = cipher.RSA_decrypt_str(private_key, encrypted_string)
+    # checking if the decrypted string is the same as the original
+    assert decrypted_string == "Hello World"
+    print ("test complete; RSA methods passed")
+    print ("testing AES methods")
+    # creating a test file
+    test_file = open("test_file.txt", "w")
+    test_file.write("Hello World")
+    test_file.close()
+    # encrypting the file
+    cipher.AES_encrypt_file("test_file.txt")
+    # decrypting the file
+    cipher.AES_decrypt_file("test_file.txt")
+    # checking if the file is decrypted
+    with open("test_file.txt", "r") as f:
+        assert f.read() == "Hello World"
+    # encrypting a string
+    encrypted_string, iv, key = cipher.AES_encrypt_string(b"Hello World")
+    # decrypting the string
+    decrypted_string = cipher.AES_decrypt_string(encrypted_string, key, iv)
+    # checking if the decrypted string is the same as the original
+    assert decrypted_string.decode() == "Hello World"
+    print ("test complete; AES methods passed")
+    print ("testing Blowfish methods")
+    # using prev test file
+    # encrypting the file
+    cipher.BLO_encrypt_file("test_file.txt")
+    # decrypting the file
+    cipher.BLO_decrypt_file("test_file.txt")
+    # checking if the file is decrypted
+    with open("test_file.txt", "r") as f:
+        assert f.read() == "Hello World"
+    # encrypting a string
+    encrypted_string, iv, key = cipher.BLO_encrypt_string(b"Hello World")
+    # decrypting the string
+    decrypted_string = cipher.BLO_decrypt_string(encrypted_string, key, iv)
+    # checking if the decrypted string is the same as the original
+    assert decrypted_string.decode() == "Hello World"
+    print ("test complete; Blowfish methods passed")
+    print ("Testing Fernet methods")
+    # using prev test file
+    # encrypting the file
+    cipher.FerNet_encrypt_file("test_file.txt")
+    # decrypting the file
+    cipher.FerNet_decrypt_file("test_file.txt")
+    # checking if the file is decrypted
+    with open("test_file.txt", "r") as f:
+        assert f.read() == "Hello World"
+    # encrypting a string
+    encrypted_string = cipher.FerNet_encrypt_string(b"Hello World")
+    # decrypting the string
+    decrypted_string = cipher.FerNet_decrypt_string(encrypted_string)
+    # checking if the decrypted string is the same as the original
+    assert decrypted_string.decode() == "Hello World"
+    print ("test complete; Fernet methods passed")
+    print ("All tests completed | All tests passed")
