@@ -1,7 +1,7 @@
 #! /usr/bin/python3.9
 from setuptools import setup, find_packages
 from Cython.Build import cythonize
-VERSION = "0.5.1"
+VERSION = "0.6.1"
 DESCRIPTION = "A module for encryption and information security operations."
 setup(
     name= "better cryptography",
@@ -14,7 +14,7 @@ setup(
     packages = find_packages(),
     install_requires = ['pycryptodome', "rsa", "cryptography"],
     ext_modules=cythonize(["better_cryptography/file_encryption.pyx"]),
-    keywords=["python", "encryption", "AES", "information security"],
+    keywords=["python", "encryption", "AES", "ciphers", "encryption alogorithms", "information security"],
     classifiers= [
         "Development Status :: 3 - Alpha",
         "Intended Audience :: Education",
